#################################################################################
#
#  Licensed Materials - Property of IBM
#  Restricted Material of IBM
#  IBM Rational Test RealTime
#  (c) Copyright IBM Corp. 2003-2014 All Rights Reserved.
#
#  US Governement Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#################################################################################

##########################################################################
#                                                                        # #
# Rational Test RealTime                                                 #   #
#                                                                        #######
#                                                                              #
#                                                                              #
#  Produce products.h from $ATLTGT/lib/products_model.h                        #
#  Compile Target Package object in $INTDIR                                    #
#                                                                              #
#                                                                              #
# Required global variables:                                                   #
#   $TARGETDIR    : directory of the Target Package                            #
#   $SCRIPTDIR    : directory of scripts                                       #
#   $INTDIR       : intermediate files directory                               #
#   $OBJEXT       : object extension                                           #
#   $ATUMODE      : 1 if use ATU                                               #
#   $ATSMODE      : 1 if use ATS                                               #
#   $ATOMODE      : 1 if use ATO                                               #
#   $ATCMODE      : 1 if use ATC                                               #
#   $ATTMODE      : 1 if use ATT                                               #
#                                                                              #
################################################################################
#$Id: Products.pl,v 1.169.4.11 2011/03/18 13:24:58 rfrance Exp $
################################################################################

use File::Copy;

require $TARGETDIR . "/cmd/atl_func.pl";
require $TARGETDIR . "/cmd/atl_cc.pl";
require $TARGETDIR . "/cmd/atl_link.pl";
require $SCRIPTDIR . "monitor.pl";
# Debug mode:
$PRODUCTS_DEBUG = 0;

################################################################################
# TargetPackageIsToBeCompiled
# Arguments:
#   $tp_obj : TP.o(bj)
#   $tmp_header : temporary products header file
#   $header : last generated products header file
# Return value:
#   0 if TP.o(bj) is up to date
#   1 if TP.o(bj) is to be recompiled
################################################################################
sub TargetPackageIsToBeCompiled #($$$)
{
  my ($tp_obj,$tmp_header,$header) = @_;

  print "PWD:",GetCurrentDir(),"\n" if($ENV{ATTOLSTUDIO_VERBOSE});
  # TP.o(bj) exists ?
  if (! -f $tp_obj) {
    return (1);
  }

  # Has customizable file been modified ?
  my $custom = "$TARGETDIR/lib/custom.h";
  if (IsOlderThan($tp_obj,$custom)) {
    return (1);
  }

  # Has last generated products header file been erased ?
  if (! -f $header) {
    return (1);
  }

  # Has products switched been modified ?
  if (FileDiff($tmp_header,$header)) {
    return (1);
  }

  # Else: TP.o(bj) is up to date
  return (0);
}

################################################################################
# MakeCProductsHeaderFile
# Arguments:
#   $model = model for products header file
#   $tmp_header = temporary products header file
# Return value:
#   none
################################################################################
sub MakeCProductsHeaderFile #($$)
{
  my ($model,$tmp_header) = @_;

  if(-f "$TARGETDIR/lib/products.h"){
	die &PerlTranslate("Products.pl", "!!! A file called products.h has been found in the directory %1/lib. Please remove or rename this file to ensure the correct compilation of the TDP\n", "%1 is the TARGET directory",$TARGETDIR);
  }
  if(($Ini{RTRT_BUFFER_SIZE}) && ($Ini{RTRT_BUFFER_SIZE}<$ATL_BUFFER_SIZE)) {
	$ATL_BUFFER_SIZE=$Ini{RTRT_BUFFER_SIZE};
	print &PerlTranslate("Products.pl", "Result buffer size reduced to %1 chars\n", "%1 is the number of chars", $ATL_BUFFER_SIZE);
  }
  # Let's open in and out files
  open(PRODUCTS_MODEL, "<$model") || die &PerlTranslate("Products.pl", "cannot open %1: %2\n", "%1 is the name of the file and %2 is the reason given by the Operating System", $model, $!);
  open(PRODUCTS_TMP, ">$tmp_header") || die &PerlTranslate("Products.pl", "cannot create %1: %2\n", "%1 is the name of the file and %2 is the reason given by the Operating System", $tmp_header, $!);

  print (PRODUCTS_TMP "/* TDP: $Ini{'NAME'} ($VERSION) */\n/* ATLTGT: $TARGETDIR */\n");
  print (PRODUCTS_TMP "/* Ini File used:\n  " . $INIFILE . " */\n\n");
  print (PRODUCTS_TMP "/* Node Name: $TEST_NAME (",GetDate(),") */\n\n");
  print (PRODUCTS_TMP "/* Preprocessor macro definitions:\n  ");
  foreach (@GENERATION_CONDITION) {
    print (PRODUCTS_TMP $_ . " ");
  }
  print (PRODUCTS_TMP "*/\n\n");
  print (PRODUCTS_TMP "/* Preprocessor options:\n  " . $CPPFLAGS . " */\n\n");
  print (PRODUCTS_TMP "/* Compiler flags:\n  " . $CFLAGS . " */\n\n");

  #set properly unset values
  $ATP_CALL_STACK_SIZE=6 unless($ATP_CALL_STACK_SIZE >0);

  # Parse in-file to change @<keyword>@ lines with Studio node settings
  while (<PRODUCTS_MODEL>) {

    if (/^\s*\#define\s+(\w+)\s+\{ (.+) }(.*)$/) {# keep space around brace
      my $setting=$1;
	  my $expr=$2;
	  my $type=$3;
      my $answer;
	  if($type=~m/YES/i){
		$answer=eval "$expr" && "ATL_YES" || "ATL_NO" ;
	  } elsif($type=~m/num/i){
		  $answer=(eval "$expr")+0;
	  } elsif($type=~m/string/i){
		  $answer='"'. (eval "$expr") .'"';
	  } elsif($type=~m/select/i){#Do not generate if no answer (selected setting)
		  $answer=(eval "$expr")+0;
		  next unless($answer);
      } else {
		  $answer=eval "$expr";
		  $answer='0' if($answer eq '');
	  }		  
      print (PRODUCTS_TMP "#define $setting $answer /* auto exp $type */\n");
	} elsif (/^\s*\@ATL\@\s+([A-Z_0-9]+)\s+(.*)$/) {
      my $setting=$1;
      my $answer="";
      if ( $setting eq "ATL_NODE_NAME" ) {
          $answer =$2;
          $answer =~ s/<string>/$NAME \/ $PROJECT_NAME/g;
          print (PRODUCTS_TMP "#define ATL_NODE_NAME ",$answer,"\n");
      # } elsif ( $setting eq "ATL_RUNTIME_FORMAT" ) {
          # $answer = "ATL_OBJ"; 
          # $answer = "ATL_DLL" if($ATL_RUNTIME_FORMAT eq "DLL");
          # $answer = "ATL_LIB" if($ATL_RUNTIME_FORMAT eq "LIB");
          # print (PRODUCTS_TMP "#define ATL_RUNTIME_FORMAT ",$answer,"\n");
      } elsif ( $setting eq "ATL_TRACES_FILE" ) {
          my $answerModel=$2;
          if ($ATL_TRACES_FILE ne "") {
	     #print "$ATL_TRACES_FILE has been set on attolcc command line\n";
             $answer = $ATL_TRACES_FILE;
          } else {
             $answer = $ATL_OUT_FILE; # Always set by Studio or scripts or plugin
             if( defined $Ini{ATL_TRACES_FILE} ){
		my $name=$Ini{ATL_TRACES_FILE};
		if($name=~ m/[\\\/]$/){ #it is a directory not a file
			my ($d,$n,$e) = SplitFileName($answer);
			$name.=$n.'.'.$e;
			$name=~s,\\\\,\\,g;
		}
		$answer=$name;
             } elsif ( $Ini{'NOPATH'} || $Ini{'NOPATHOUT'} ) {
                my ($dir, $base, $ext) = SplitFileName ($answer);
                $answer = "$base.$ext";
	     } elsif ($Ini{'ABSPATHOUT'} ) {
		$answer=absolutePath($answer);
             }
          }
          $answerModel =~ s/<string>/$answer/g;
          $answer = $answerModel;
          $answer =~ /.*\"(.*)\"/;
          print (PRODUCTS_TMP "#define ATL_TRACES_FILE_LEN ", length($1), "\n");
          #$answer =~ s/\\/\\\\/g;
          $answer = LocaleStringReplace($answer,"\\\\","\\\\");
          print (PRODUCTS_TMP "#define ATL_TRACES_FILE ", $answer, "\n");
          RegisterGenFiles("RES $1");
      } elsif ( $setting eq "ATL_HOST_ADDRESS" ) {
		  if(($ADDRESS eq "127.0.0.1") || ($ADDRESS eq "::1") || ($ADDRESS eq "")){
			  if($ENV{ADDRESS} ne '') {
					$ADDRESS=$ENV{ADDRESS};
			  } elsif(defined $Ini{ADDRESS}) {
					$ADDRESS=$Ini{ADDRESS};
			  } else {
				  $ADDRESS=hostinet($ADDRESS);
			  }
	      }
		  my $ADDRESSNS=$ADDRESS; 
		  if($ADDRESS eq "localhost"){
			  $ADDRESSNS='127,0,0,1';
            } elsif($ADDRESS=~ /:/){
			  print (PRODUCTS_TMP "#define ATL_HOST_ADDRESS_IPv6\n");
			  $ADDRESSNS='127,0,0,1';# not good but not nothing
            } else {
			  $ADDRESSNS=~s/\./,/g;
            }
		  print (PRODUCTS_TMP "#define ATL_HOST_ADDRESS \"", $ADDRESS, "\"\n");
		  print (PRODUCTS_TMP "#define ATL_HOST_ADDRESS_NS ", $ADDRESSNS, "\n");
      } elsif ( $setting eq "ATL_HOST_ADDRESS_PORT" ) {
	    my $uport=$UPLOADER_PORT;
	    $uport=7777 unless($uport);
	    $uport=$Ini{UPLOADER_PORT} if(($uport == 7777) && defined $Ini{UPLOADER_PORT});
	    $uport=$ENV{UPLOADER_PORT} if(($uport == 7777) && $ENV{UPLOADER_PORT} ne '');
	    print (PRODUCTS_TMP "#define ATL_HOST_ADDRESS_PORT ", $uport, "\n");
      }
    } elsif (/^\s*\@TST\@\s+([A-Z_0-9]+)\s(.*)$/) {
      my $setting=$1;
      my $answer="";
      if ( $setting eq "ATU_OPTIM_INCLUDE" ) {
        if($tmp_header=~/^(.*)products/) {
      	    my $attol_cfg=$1."attol_cfg.h";
      	    if(-f $attol_cfg) {
				print (PRODUCTS_TMP "#include \"attol_cfg.h\" \n");
      	    }
		}
      }elsif ( $setting eq "ATV_OPTIM_INCLUDE" ) {
        my ($otmDir,$otmBase,$otmExt)=SplitFileName($ATV_INTERMEDIATE_FILE);
       	my $otmName=$otmBase.'.h';
     	if(-f $otmDir.$otmName) {
      	        $otmName=$otmDir.$otmName unless($Ini{'NOPATH'});
		print (PRODUCTS_TMP "#include \"$otmName\" \n");
      	}
      } elsif ( $setting eq "ATS_THREAD_FUNCTION" ) {
		$answer = $ATS_THREAD_FUNCTION;
		$answer ='ATL_Start' if($answer eq '');
		print (PRODUCTS_TMP "#define ATS_THREAD_FUNCTION ",$answer, "\n");
		print (PRODUCTS_TMP "#define ATS_THREAD_TABLE ${answer}_Table\n");
		print (PRODUCTS_TMP "#define ATS_THREAD_TABLELEN ${answer}_TableLen\n");
      } else {
          print (PRODUCTS_TMP "/* #define $1 $2 */\n");
          print ("/* #define $1 $2 */\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      }
    # } elsif (/^\s*\@RTA\@\s+([A-Z_0-9]+)\s(.*)$/) {
      # my $setting=$1;
      # my $answer="";
      # if ( $setting eq "ATC_DUMP_DRIVER" ) {
          # $answer = "ATL_NO";
          # $answer = "ATL_YES" if($ATC_DUMP_DRIVER ==1 || lc($ATC_DUMP_DRIVER) eq 'merged');
          # print (PRODUCTS_TMP "#define ATC_DUMP_DRIVER ",$answer,"\n");
      # } 
      
    } elsif (/^\s*\/\*\s*TDP .Ini.RUNTIME_DEFINES/) {
	  foreach $tdpdef (split(/[,;]/,$Ini{RUNTIME_DEFINES})){
		$tdpdef=~ s!=! !;
		print (PRODUCTS_TMP "#define $tdpdef \n");
	  }
    } elsif (/^\s*\/\*\s*TDP .ENV.RUNTIME_DEFINES/) {
	  foreach $tdpdef (split(/[,;]/,$ENV{RUNTIME_DEFINES})){
		$tdpdef=~ s!=! !;
		print (PRODUCTS_TMP "#define $tdpdef \n");
	  }
    } else {
      print PRODUCTS_TMP;
    }

  }

  # That's all folks
  close(PRODUCTS_TMP);
  close(PRODUCTS_MODEL);
}

################################################################################
# GetTargetPackageObject
# Arguments:
#   none
# Return value:
#   Path and filename of the object file
################################################################################
#sub GetTargetPackageObject
#{
#  my $d=$INTDIR;
#  if($ENV{'ATTOLOBJ'}) {
#     $d=$ENV{'ATTOLOBJ'};
#  }
#
#  if ( $OSNAME eq "win32" )
#    {
#      return "$d\\TP.$OBJEXT";
#    }
#  else
#    {
#      return "$d/TP.$OBJEXT";
#    }
#}
################################################################################
# GetTargetPackagesLib
# Arguments:
#   none
# Return value:
#   Path and filename of the Lib file
################################################################################
sub GetTargetPackageLib #($)
{
  my $TPext = shift;
  my $d=$INTDIR;
  if($ENV{'ATTOLOBJ'}) {
     $d=$ENV{'ATTOLOBJ'};
  }
 #------------- modif for startup code ---------------
    if($Ini{STARTUP_CODE}){
      foreach my $startup_src (split(/[,;]/,$Ini{STARTUP_CODE})) {
	my ($dir,$base,$ext) = SplitFileName ($startup_src);
	my $startup_object="$INTDIR/$base.$OBJEXT";
	#put this object file on the first position
	  unless(grep(/[\\\/]$base\.$OBJEXT$/i,@OBJECTS)) {
		unshift(@OBJECTS, $startup_object);
	  }
      }
    }
#------------- end of modif for startup code ---------------

  if ( $OSNAME eq "win32" )
    {
      return "$d\\TP.$TPext";
    }
  else
    {
      return "$d/TP.$TPext";
    }
}

################################################################################
# MakeTargetPackageObject
# Arguments:
#   none
# Return value:
#   none
################################################################################
sub MakeTargetPackageObject
{
  my $model = "$TARGETDIR/lib/products_model.h";
  my $tmp_header = "$INTDIR/products_tmp.h";
  my $header = "$INTDIR/products.h";
  my $tp_odir  = $INTDIR; $tp_odir=$ENV{ATTOLOBJ} if($ENV{ATTOLOBJ});
  my $tp_obj = $tp_odir.$PATH_SEPARATOR.'TP.'.$OBJEXT;
  my $tp_lib = $tp_obj;
  my $tp_ldflags= $TP_LDFLAGS;
  my $tp_cflags = $CPPFLAGS.' '.$CFLAGS.' '.$PPFLAGS;
     $tp_cflags = $TP_CCFLAGS if($TP_CCFLAGS ne "");
  $ATL_RUNTIME_FORMAT="OBJ" if($ATL_RUNTIME_FORMAT eq "");
     $tp_lib = $tp_odir.$PATH_SEPARATOR.'TP.'.$LIBEXT if($ATL_RUNTIME_FORMAT eq "LIB");
     $tp_lib = $tp_odir.$PATH_SEPARATOR.'TP.'.$DLLEXT if($ATL_RUNTIME_FORMAT eq "DLL");

  ## Do not forget the OBJ file in output log
  my @loggedFiles=($tp_obj,$header);
  ## just do delete it from Studio
  push(@loggedFiles,$tp_lib) if($ATL_RUNTIME_FORMAT ne "OBJ");
  RegisterTmpFiles( @loggedFiles );

  # First make the temporary products header file
  MakeCProductsHeaderFile($model,$tmp_header);

  # Now decide if Target Package object is to be compiled, if so: do it !
#  if ((TargetPackageIsToBeCompiled($tp_obj,$tmp_header,$header))||($PRODUCTS_SPV)) {
  if (TargetPackageIsToBeCompiled($tp_lib,$tmp_header,$header)) {

    if ( -f $header ) {
      unlink ($header) || die &PerlTranslate("Products.pl", "unlink %1 failed\n", "%1 is the file to be unlinked", $header);
    }
    rename ($tmp_header,$header) || die &PerlTranslate("Products.pl", "rename %1 to %2 failed\n", "%1 is the original name of the file and %2 is the new name", $tmp_header, $header);

	if($MNTMODE && $ATAMODE ){
		if(@TSFFILES){
			print "masterTable.h generation ...\n";
			genMonitorTable(\@TSFFILES) ;
		} else {
			print "No tsf file found to generate masterTable.h !!!\n";
		}
	}

    # Target Package compilation:
    my $tp_src_ext = ".c";
    my $language = "C";
    if ( $TDP_LANGUAGE eq "CPP" ) {
      $tp_src_ext = ".cpp";
      $language = "CPP";
    }
    my $tp_src = $TARGETDIR . $PATH_SEPARATOR . "lib" . $PATH_SEPARATOR . "TP$tp_src_ext";
    my @tp_includes = ( "$INTDIR", $TARGETDIR . $PATH_SEPARATOR . "lib", @COMPLETE_INCLUDE_DIR_LIST );
    print &PerlTranslate("Products.pl", "Compiling %1 using %2 containing C/C++ TDP Library settings...\n", "%1 is the file to be compiled and %2 is the file to be used", $tp_src, $header) if ($ENV{'ATTOLSTUDIO_VERBOSE'});
    $status = atl_cc ( $language, $tp_src, $tp_obj, $tp_cflags,
                       \@GENERATION_CONDITION, \@tp_includes );
    EndStep($status);
    if($status != 0) {
      print &PerlTranslate("Products.pl", "Target Package compilation failed\n");
      unlink ($tp_obj);
	  atl_cpp ( $language, $tp_src, "$INTDIR/TP.i", $tp_cflags,
                       \@GENERATION_CONDITION, \@tp_includes ) if ($ENV{'ATTOLSTUDIO_VERBOSE'});
      return (1);
    }
    if($ATL_RUNTIME_FORMAT ne "OBJ") {
      my @OBJs = ($tp_obj);
      my @ADDlibs = ();
      $status = atl_lib ( $tp_lib, \@OBJs, $tp_ldflags,
                           \@LIBRARY_DIR_LIST, $LIBRARIES, \@ADDlibs, $ATL_RUNTIME_FORMAT );
      EndStep($status);
      if($status != 0) {
         print &PerlTranslate("Products.pl", "Target Package Library compilation failed\n");
         unlink ($tp_lib);
         return (1);
       }

    }
    #make startup if required
    if($Ini{STARTUP_CODE_GENERATE}){ startup_code_generate(); }#This method must be part of the TDP
    if($Ini{STARTUP_CODE}){
      foreach my $startup_src (split(/[,;]/,$Ini{STARTUP_CODE})) {
	print &PerlTranslate("Products.pl", "Need startup file %1\n", "%1 is the file to be compiled", $startup_src);
	my @inc_dir_list=@COMPLETE_INCLUDE_DIR_LIST;
	my ($dir,$base,$ext) = SplitFileName ($startup_src);
	my $startup_object="$INTDIR/$base.$OBJEXT";
	if($dir){
		$dir=~s,[\\\/]$,,;
		unshift(@inc_dir_list,$dir) ;
	} else {
		$startup_src="$TARGETDIR/lib/$base.$ext";
	}
	#put this object file on the first position
	unless(grep(/[\\\/]$base\.$OBJEXT$/i,@OBJECTS)) {
		unshift(@OBJECTS, $startup_object);
		RegisterTmpFiles($startup_object);
		unless(-f $startup_object){
		print &PerlTranslate("Products.pl", "Compiling startup file %1\n", "%1 is the file to be compiled", $startup_src);
			my $lang=LanguageOf($ext);
			if ( $lang eq "C" ) {
				$status = atl_cc ($lang, $startup_src, $startup_object, $tp_cflags,
					\@GENERATION_CONDITION, \@inc_dir_list);
			} elsif ( $lang eq "ASM" ) {
				my $asmFlgs=$ASMFLAGS; $asmFlgs=$Ini{DEFAULT_ASMFLAGS} unless($ASMFLAGS);
				$status = atl_asm ($lang, $startup_src, $startup_object, $asmFlgs,
					\@GENERATION_CONDITION, \@inc_dir_list);
			} else {
				print "Cannot compile startup file $startup_src (unknown extension: $ext). Assume object file\n";
				$status = 0;
			}
			EndStep($status);
			if($status) {
				print &PerlTranslate("Products.pl", "Target Package startup compilation failed\n");
				unlink ($startup_object);
				return (1);
			}
		}
	}
      }
    }
  } else {

    print &PerlTranslate("Products.pl", "up to date\n", "") if ($ENV{'ATTOLSTUDIO_VERBOSE'});
    # Remove temporary products header file
    unlink ($tmp_header);

  }

  return (0);
}

################################################################################
# MakeCompilationOptionsFile
# Arguments:
#   none
# Return value:
#   none
################################################################################
sub MakeCompilationOptionsFile
{
   $model = "$TARGETDIR/lib/options_model.h";
   $options = "$INTDIR/options.h";

   MakeCProductsHeaderFile ($model, $options);
   return 0;
}


1;
##
################################################################################

