#################################################################################
#
#  Licensed Materials - Property of IBM
#  Restricted Material of IBM
#  IBM Rational Test RealTime
#  (c) Copyright IBM Corp. 2005-2017 All Rights Reserved.
#
#  US Governement Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#################################################################################
# Debug mode:
$REMOTE_DEBUG = 0;

# These remote function help to automate the execution of the application on a remote machine
#This file allows the execution on remote machine when the remote machine is a unix/linux like, OS.
#This file does not include functions allowing to compile on remote machine. If you need to also compile on the remote machine contact the support or have a look at cxlinuxgnu.xdp
#This file include 3 different ways of disk sharing with the remote machine. atl_exec1, atl_exec2 and atl_exec3
#So the complete content of the TDP execution function can be replaced by atl_exec2($exe,$out,$parameterd);

#list of required Basic settings keys and when they are required. e.g: (2,3) means required by atl_exec2 and atl_exec3 only
#REMOTE_MACHINE		(all) IP address or networkName of the remote machine used to execute
#REMOTE_USER		(all) userName to login on remote machine. remove or keep empty  if Win32 %USERNAME% or $USER can be used instead
#REMOTE_PASSWD		(all) PassWord to login on remote machine. It is used to replace the keyword _PassWd_ when present in the REMOTE_CONNECTION command
#REMOTE_LAUNCHER	(all)(optional) Name of the executable to launch on the remote machine to execute the application. keep it empty or not set if the application is native
#REMOTE_CONNECTION	(all) mode used to connect to the remote machine. known mode are: ssh,rsh,plink,putty,telnet.pl (default is rsh) on Win32 ssh2 is used instead of ssh.
#				if no known mode apply then the complete command line can be entered using variables:_PassWd_,_user_,_machine_ that will be replaced appropiate value at exec time
#				e.g. for connection from Linux host to an other machine: sshpass -p _PassWd_ ssh -l _user_@_machine_
#SHARED_REMOTE_DIR	(1,2) specify the project absolute path of the shared disk space on the remote machine. (from remote machine point of view)
#SHARED_HOST_DIR	(1,2) specify the project absolute path of the shared disk space on the machine hosting Test RealTime. (from host machine point of view)

#NOPATH or NOPATHOUT	(2,3) NOPATH=On or NOPATHOUT=On

#SHARED_REMOTE_DIR 	(3) specify the project absolute path of working directory on the remote machine. (from remote machine point of view)
#REMOTE_FTP_USER 	(3) specify the userName used at ftp connection if differ from REMOTE_USER above
#REMOTE_FTP_PASSWD 	(3) specify the password used at ftp connection if differ from REMOTE_PASSWD above
#REMOTE_FTP_DIR 	(3) optional specify the FTP destination directory for all copied file when SHARED_REMOTE_DIR is not accessible from FTP

#for remote compilation only. Whatever if the system includes are shared or copied:
#LOCAL_SYS_INCL		(remote compilation) specify the system include absolute path of the disk space on the remote machine. (from remote machine point of view)
#REMOTE_SYS_INCL	(remote compilation) specify the system include absolute path on the Host machine where include system can be found. (from host machine point of view)
#			remote TDPs are written into $SHARED_REMOTE_DIR/TestRealTime/$VERSION/targets
#additional check into exec func
# die "SHARED_HOST_DIR key into TDP basic settings For All is not set with an existing directory\n" unless(-d $Ini{SHARED_HOST_DIR});

sub Remote_Host 		# returns remote host name
{
    if (defined $ENV{'REMOTE_MACHINE'}) {
	return $ENV{'REMOTE_MACHINE'};	# use env. var if defined
    } else {
	if (defined $ENV{'REMOTE_HOST'}) {
		return $ENV{'REMOTE_HOST'};	# use env. var if defined
	} else {
		return $Ini{'REMOTE_HOST'} if (defined $Ini{'REMOTE_HOST'});	# else fallback to basic setting
	}
	return $Ini{'REMOTE_MACHINE'};	# else fallback to basic setting
    }
}

sub Remote_User 		# returns remote user name
{
	if (defined $ENV{'REMOTE_USER'}) {
		return $ENV{'REMOTE_USER'};	# use env. var if defined
	} elsif (defined $Ini{'REMOTE_USER'}) {
		return $Ini{'REMOTE_USER'};	# else fallback to basic setting if set
	} else {
		return ( ($OSNAME eq "win32") ? $ENV{'USERNAME'} : $ENV{'USER'});	# else use local user name
	}
}

sub Remote_PassWd 		# returns remote user PassWd
{
	if (defined $ENV{'REMOTE_PASSWD'}) {
		return $ENV{'REMOTE_PASSWD'};	# use env. var if defined
#	} elsif ( $Ini{'REMOTE_PASSWD'} ne "") {
#		return $Ini{'REMOTE_PASSWD'};	# else fallback to basic setting if set
#	} elsif (defined $Ini{'REMOTE_FTP_PASSWD'}) {
#		return $Ini{'REMOTE_FTP_PASSWD'};	# else use FTP PassWd if set
#	} else {
#		return Remote_User();	# else use UserName as passWord use _user_ as password
	}
	return $Ini{'REMOTE_PASSWD'};	# else fallback to basic setting
}

sub exec_status
{
my $log=shift;
my $remote_host = shift;
my $status=-9;
if(open(IN,"<$log"))
    {
	while(<IN>)
	{
		chomp;
		if (/^status=\s*(\d+)\s*$/) {
			$status=$1 unless($status>0);
		}
		print "[$remote_host] $_\n";
	}
	close(IN);
	if($REMOTE_DEBUG){
		rename( $log,$log.'.bak');
	} else {
		unlink ( $log );
	}
    } else {
        print "$log file not found on host\nNo execution status available (set to -9 as default)\n";
    }
return ($status);

}
# this methode execute the remote shell command using the method defined into Basic settings REMOTE_CONNECTION key
# for ssh, rsh the parameter are completed with Remote_Host and Remote_User if the passwd is required then the _PassWd_ keyword
# must be placed in the method line. It will be replaced by the PassWd value from REMOTE_PASSWD environment variable or basic setting
# for putty the parameters are completed with Remote_Host and Remote_User if not already on the line  and if no -load option
# for tdpRemote_Shell the sub tdpRemote_Shell from TDP> build settings>additional perl function is called see example of this method at the end.
# for telnet.pm ... the sub telnet is called
# for any other method the complete command must be part of the key value except the remote command:
# if the method include the file name remote_cmd.sh the cmd list is then written in this file otherwise cmd list follows the line as the last parameter
sub Remote_Shell 		#(@) returns remote shell execution status
{
	my $cmdList=shift;
	my $method = $Ini{'REMOTE_CONNECTION'};
	my $remote_host = Remote_Host();
	my $remote_user = Remote_User();
	my $remote_pass = Remote_PassWd();
	my $remote_shell =  "";
	my $status=1;
	$method=~s/_PassWd_/$remote_pass/i;
	$method=~s/_user_/$remote_user/i;
	$method=~s/_(host|machine)_/$remote_host/i;
	$REMOTE_DEBUG && print "Remote_Shell > method:$method,cmd: @$cmdList\n";
	if ($method=~ m/^(ssh|plink)/) {
		if ($OSNAME eq "win32") {
			$remote_shell =  "$method $remote_user\@$remote_host";	# Windows: assume SSH Communications' SSH client
		} else {
			$remote_shell =  "$method -n $remote_user\@$remote_host";	# UNIX/Linux: assume OpenSSH SSH client
		}
	} elsif ($method=~ m/^rsh/) {
		if ($OSNAME eq "win32") {
			$remote_shell =  "$method $remote_host -l $remote_user";
		} else {
			$remote_shell =  "$method -n -l $remote_user $remote_host";
		}
	} elsif ($method=~ s/^telnet\.p[lm]/$remote_host/) {
		$REMOTE_DEBUG && print "##>method> $method\n";
		my @args=split(/ /,$method);
		push(@args,@$cmdList);
		return telnet(@args);
	} elsif ($method=~ m/^tdpRemote_Shell/i) {
		$REMOTE_DEBUG && print "##>method> $method\n";
		return tdpRemote_Shell($cmdList);
	} elsif($method=~m/^putty/i){
		#putty -telnet|-serial|-ssh -2 -pw _PassWd_ [-load <sessionName>]
		#-2 means ssh2; -6 means IPv6; -v increase verbosity;
		if ($OSNAME eq "win32") {
			if($method=~m/(-load |\@)/){
				$remote_shell =  "$method -m remote_cmd.sh";	# Windows: assume SSH Communications' SSH client
			} else {
				$remote_shell =  "$method $remote_user\@$remote_host -m remote_cmd.sh";	# Windows: assume SSH Communications' SSH client
			}
		} else {
			print "$method not yet implemented in $ENV{TESTRTDIR}/lib/scripts/remoteFcts.pl for this machine\n";
		}
	} else {
		$remote_shell =  $method;
	}

	if($remote_shell=~m/remote_cmd.sh/){
		open(CMD,">remote_cmd.sh") || print "unable to create the file remote_cmd.sh\n";
		print CMD join("\n",@$cmdList),"\n";
		close CMD;
	} else {
		$remote_shell .=' "'.join(' ; ',@$cmdList).' "';
	}
	print join("\n$remote_user\@$remote_host> ",($Ini{'REMOTE_CONNECTION'},@$cmdList)),"\n" if ($ENV{'ATTOLSTUDIO_VERBOSE'});
	$status=system($remote_shell);
	return ($status);
}
our $Remote_SystemP_log=1;
sub Remote_SystemP
{
my @Command=@_;
my $log="remote_err.$$.log";
my $remote_dir=Remote_Path( GetCurrentDir());
foreach $cmd (@Command){
   $cmd.=" 2>>$log" if($Remote_SystemP_log && ($cmd !~m/ 2>/));
}
my @remote_cmd=("cd $remote_dir",@Command,"echo status=\$? >> $log");
my $status=Remote_Shell(\@remote_cmd);
$status=exec_status($log,$log) unless($status);
return $status;
}

sub Remote_Path #($)	# returns local paths translated to remote paths
{
	my $remote_path = shift;
	$remote_path=~s/\\+/\//g; # map remaining '\'s to '/'s
	$remote_path=~s/\/+$//;   # remove trailling /
	
	my $targetsdir=$TARGETDIR; $targetsdir=~s/\\/\//g; $targetsdir=~s!/targets/.+$!/targets!;
	$targetsdir=~s!\/+!\[\\\\\\\/\]!g;
	my $remote_targetsdir="$Ini{SHARED_REMOTE_DIR}/TestRealTime/$VERSION/targets";
	my $local_sys_share=$Ini{LOCAL_SYS_INCL}; $local_sys_share=~s/\/+$//; $local_sys_share=~s![\\\/]+!\[\\\\\\\/\]!g;
	my $local_user_share=$Ini{SHARED_HOST_DIR}; $local_user_share=~s/\/+$//; $local_user_share=~s![\\\/]+!\[\\\\\\\/\]!g;

	$remote_path=~s/^$local_user_share/$Ini{SHARED_REMOTE_DIR}/g if($local_user_share ne '');	# map local share to remote 'user' dir
	$remote_path=~s/^$local_sys_share/$Ini{REMOTE_SYS_INCL}/g if($local_sys_share ne '');		# map local share to remote 'sys' dir
	$remote_path=~s/^$targetsdir/$remote_targetsdir/g if($targetsdir ne '');			# map TDP target dir if needed
	if($remote_path=~m/_user_/){
		my $usr=Remote_User();
		$remote_path=~s/_user_/$usr/g;
	}
	$remote_path=~s/\\/\//g;					# map remaining '\'s to '/'s

	return ($remote_path);
}

# works from Windows only if the SHARED_HOST_DIR uses the windows path format(ie Z:\...) with a 'net use' to map the drive to network machine/path
sub atl_exec1 #($$$)	# host and remote share the project directory as disk space
{					# SHARED_REMOTE_DIR specify the remote path of the shared directory.
					# SHARED_HOST_DIR specify the local path of the shared directory
my ( $exe,$out,$parameters )= @_;
#	my ($d,$b,$e)=SplitFileName($exe);
	my $remote_dir=Remote_Path( GetCurrentDir());
	my $log="remote.$$.log";
	my $remote_exe = $exe; $remote_exe=~s/\\+/\//g;
	my $launcher = $Ini{REMOTE_LAUNCHER};

	my @remote_command;
#	push(@remote_command,"cd $Ini{'SHARED_REMOTE_DIR'}");
	push(@remote_command,"cd $remote_dir");
	push(@remote_command,"chmod a+rx ./$remote_exe");
	push(@remote_command,"export LD_LIBRARY_PATH=\$LD_LIBRARY_PATH:$remote_dir");
      if($EXECMODE ne 'DEBUG'){
	push(@remote_command,"$launcher ./$remote_exe $parameters");
	push(@remote_command,"echo status=\$? >> $log"); # 2>&1
      }
	print "EXECUTE remote command (exec)...\n";
	$status=Remote_Shell(\@remote_command);
	WaitFromUser() if($EXECMODE eq 'DEBUG');
	print "remote command failed status $status\n" if($status);
        print "\n", "=" x 12,  GetDate(), "=" x 12 ,"\n";

	return exec_status($log,Remote_Host());
}

sub atl_exec2 #($$$)	# host and remote share a fixed work directory as disk space (basic setting NOPATHOUT/NOPATH is required)
{			# SHARED_REMOTE_DIR specify the tmp work directory on the remote machine.
			# SHARED_HOST_DIR specify the work directory from local/host machine.
			# REMOTE_LAUNCHER can be an application used to execute the $exe (can be empty)
my ( $exe,$out,$parameters )= @_;
	my $cwd = cwd();
	my $log="remote.$$.log";
    my ($exedir,$exebase,$exeext)= SplitFileName ($exe);
	my $remote_exe = $exebase.'.'.$exeext;
	my ($outdir,$outbase,$outext)= SplitFileName ($out);
	my $remote_out = $outbase.'.'.$outext;
	my $remote_dir = $Ini{SHARED_REMOTE_DIR};
	if($remote_dir=~m/_user_/){
		my $usr=Remote_User();
		$remote_dir=~s/_user_/$usr/g;
	}
	my $local_dir  = $Ini{SHARED_HOST_DIR};
	my $launcher = $Ini{REMOTE_LAUNCHER};

    print "copy $exe to $local_dir/$remote_exe\n";
	copy($exe,$local_dir.'/'.$remote_exe);

	my @remote_command;
	push(@remote_command,"cd $remote_dir");
	push(@remote_command,"chmod a+rx ./$remote_exe");
	push(@remote_command,"export LD_LIBRARY_PATH=\$LD_LIBRARY_PATH:$remote_dir");
    push(@remote_command,split(/;/,$Ini{REMOTE_PRESET_ENVIR})) if(defined $Ini{REMOTE_PRESET_ENVIR} && $Ini{REMOTE_PRESET_ENVIR} ne '');
	if($EXECMODE ne 'DEBUG'){
	push(@remote_command,"$launcher ./$remote_exe $parameters");
	push(@remote_command,"echo status=\$? >> $log"); # 2>&1
      }
	print "EXECUTE remote command (exec)...\n";
	$status=Remote_Shell(\@remote_command);
	WaitFromUser() if($EXECMODE eq 'DEBUG');

	copy($local_dir.'/'.$log,$log);
	copy($local_dir.'/'.$remote_out,$out);
	print " !!! results file not found !!!\n" unless(-f $out);
	unlink($local_dir.'/'.$log,$local_dir.'/'.$remote_out,$local_dir.'/'.$remote_exe);
	print "remote command failed status $status\n" if($status);
        print "\n", "=" x 12,  GetDate(), "=" x 12 ,"\n";

	return exec_status($log,Remote_Host());
}


sub atl_exec3 #($$$)	# host and remote does not share any disk space (basic setting NOPATHOUT/NOPATH is required)
{			# SHARED_REMOTE_DIR specify the tmp work directory on the remote machine.
			# REMOTE_FTP_USER specify the userName used at ftp connection, otherwise REMOTE_USER or $USERNAME is used
            # REMOTE_FTP_PASSWD specify the password used at ftp connection
			# REMOTE_FTP_DIR specify the ftp destination directory for all copied file when SHARED_REMOTE_DIR is not accessible via FTP
			# once copied the rsh/ssh command copy the file into its own directory (working directory)
			# then execute it
			# the FTP account must have read access on the exec/rsh account and vice-versa
#FTP account and USER(exec) account on the remote machine may be the same if you modify the .rhosts file as:
#add the the host machine name or the host machine IP followed by a blanc space and '+' to authorize the host machine to connect to the
#remote machine with and without passwd. So the host machine could connect to the remote via ftp with passwd and rsh without passwd.
#Both using the same account and so the same directory.
#Note that /etc/hosts.equiv should have the same content as well.
my ( $exe,$out,$parameters )= @_;
	my $remote_host = Remote_Host();
	my $remote_user = Remote_User ();
	my $launcher = $Ini{REMOTE_LAUNCHER};
	my $remote_ftp_user = $Ini{'REMOTE_FTP_USER'};
	$remote_ftp_user=$remote_user unless(defined $Ini{'REMOTE_FTP_USER'});
	my $cwd = cwd();
	my $sendStatus;
	my $log="remote.$$.log";
             my ($exedir,$exebase,$exeext)= SplitFileName ($exe);
	my $remote_exe = $exebase.'.'.$exeext;
	my ($outdir,$outbase,$outext)= SplitFileName ($out);
	$outdir =~ s/[\\\/]+$//;
	my $remote_out = $outbase.'.'.$outext;
	my $remote_dir = $Ini{SHARED_REMOTE_DIR};
	my $remote_ftp_dir=$remote_dir;
	   $remote_ftp_dir=$Ini{'REMOTE_FTP_DIR'} if($Ini{'REMOTE_FTP_DIR'});
	my $remote_dir_is_ftp=1 if($remote_dir eq $remote_ftp_dir);
	if($remote_ftp_dir=~m/_user_/){
		my $usr=Remote_User();
		$remote_ftp_dir=~s/_user_/$usr/g;
	}
	if($remote_dir=~m/_user_/){
		my $usr=Remote_User();
		$remote_dir=~s/_user_/$usr/g;
	}

	my $remote_pass=$Ini{'REMOTE_FTP_PASSWD'};
	$remote_pass=Remote_PassWd() unless(defined $Ini{'REMOTE_FTP_PASSWD'});
	$remote_pass=~s/_user_/$remote_user/i;

	#send the executable on the remote machine
    $cmd="";
	open SCR,">send.txt";
  if(defined $Ini{REMOTE_TRANSFERT}){ #sftp _user_@_machine_
	foreach my $str (split(/;/,$Ini{REMOTE_TRANSFERT})){
            $str=~s/_PassWd_/$remote_pass/i;
			$str=~s/_user_/$remote_user/g;
			$str=~s/_(host|machine)_/$remote_host/ig;
			$str=~s/_script_/send.txt/ig;
            
			$cmd="$str";
	}
  } else {
       if($OSNAME eq "win32") {
          $cmd="ftp -s:send.txt";
            print SCR "open $remote_host\n";
       } else {
           $cmd="./send.txt";
            print SCR "ftp -v -n $remote_host<<EOF\n";
            print SCR "user ";
       }
	print SCR $remote_ftp_user ,"\n";
	print SCR $remote_pass ,"\n";
  }
	print SCR "mkdir $remote_ftp_dir\n";
	print SCR "cd $remote_ftp_dir\n";
	print SCR "binary\n";
	print SCR "put $exe\n";
	print SCR "bye\n";
	close SCR;
	if(REMOTE_DEBUG){ open SCR,"send.txt";print "send.cmd:$_" while(<SCR>);close SCR;}
        print "\nSEND \$exe=$exe to target user $remote_ftp_user $remote_ftp_dir ...\n";
       if($OSNAME eq "win32" && !defined $Ini{REMOTE_TRANSFERT}) {
	$sendStatus=SystemP($cmd);
       } else {
    if ($OSNAME eq "win32") {
        $sendStatus=SystemP( $cmd);
    } else {
        chmod(0755,"send.txt");
        $sendStatus=SystemP( $cmd);
       }
       }
        print "\n", "=" x 12,  GetDate(), "=" x 12 ," SEND Status=$sendStatus\n";

	#execute the test
	my @remote_command;
	push(@remote_command,"cd $remote_dir");
	push(@remote_command,"rm $remote_out");

	push(@remote_command,"cp $remote_ftp_dir/$remote_exe .") unless($remote_dir_is_ftp);
	push(@remote_command,"chmod a+rx $remote_exe");
	push(@remote_command,"export LD_LIBRARY_PATH=\$LD_LIBRARY_PATH:$remote_dir");
    push(@remote_command,split(/;/,$Ini{REMOTE_PRESET_ENVIR})) if(defined $Ini{REMOTE_PRESET_ENVIR} && $Ini{REMOTE_PRESET_ENVIR} ne '');
      if($EXECMODE ne 'DEBUG'){
	push(@remote_command,"$launcher ./$remote_exe $parameters");
	push(@remote_command,"echo status=\$? >> $log"); # 2>&1
      }
	print "EXECUTE remote command (exec)...\n";
	$status=Remote_Shell(\@remote_command);
	print "remote command (exec) failed status $status\n" if($status);
        print "\n", "=" x 12,  GetDate(), "=" x 12 ,"\n";
	sleep(2);
	WaitFromUser() if($EXECMODE eq 'DEBUG');
	return 0 if(defined $Ini{RTRT_IO});# STD mode does not define. In other mode no need to recover the result file

	#get the result files from the remote machine
    $cmd="";
	open SCR,">get.txt";
  if(defined $Ini{REMOTE_TRANSFERT}){ #sftp _user_@_machine_
	foreach my $str (split(/;/,$Ini{REMOTE_TRANSFERT})){
            $str=~s/_PassWd_/$remote_pass/i;
			$str=~s/_user_/$remote_user/g;
			$str=~s/_(host|machine)_/$remote_host/ig;
			$str=~s/_script_/get.txt/ig;
            
			$cmd="$str";
	}
  } else {

       if($OSNAME eq "win32") {
          $cmd="ftp -s:send.txt";
	print SCR "open $remote_host\n";
       } else {
           $cmd="./send.txt";
	print SCR "ftp -v -n $remote_host<<EOF\n";
	print SCR "user ";
       }
	print SCR $remote_ftp_user ,"\n";
	print SCR $remote_pass ,"\n";
    }
  unless($remote_dir_is_ftp) {
	print SCR "cd ",$remote_ftp_dir ,"\n";
	print SCR "del $remote_exe\n";
  }
	print SCR "cd $remote_dir\n";
	print SCR "ls\n"; # for debug
	print SCR "del $remote_exe\n";
	print SCR "get $log\n";
	print SCR "lcd $outdir \n";
	print SCR "get $remote_out\n";
  if($remote_dir_is_ftp) {
	print SCR "del $log\n";
	print SCR "del $remote_out\n";
  }
	print SCR "bye\n";
	close SCR;
	if(REMOTE_DEBUG){ open SCR,"get.txt";print "get.txt:$_" while(<SCR>);close SCR;}
        print "GET $remote_out from target user $remote_ftp_user $remote_ftp_dir ...\n";
       if($OSNAME eq "win32") {
	$sendStatus=SystemP($cmd);
       } else {
	chmod(0755,"get.txt");
	$sendStatus=SystemP($cmd);
       }
	if(-s $out){
		unlink("send.txt","get.txt") unless($ENV{ATL_KEEP});
	} else {
		print "!!! $out not available or empty !!!\n";
	}
        print "\n", "=" x 12,  GetDate(), "=" x 12 ," GET Status=$sendStatus\n";

  unless($remote_dir_is_ftp) {
	my @remote_command;
	push(@remote_command,"cd $remote_dir");
	push(@remote_command,"rm $remote_out");
	push(@remote_command,"rm $log");#remove the log file

	print "RM remote command ...\n";
	$sendStatus=Remote_Shell(\@remote_command);
	print "RM status $sendStatus\n";
  }

	return exec_status($log,$remote_host);
}

#used for remote compilation
sub Remote_fixlines #($$)# generates from given input file an output file with '#line' directives translated from remote paths to local paths
{
	my ($in,$out)=@_;

	my $targetsdir=$TARGETDIR; $targetsdir=~s/\\/\//g; $targetsdir=~s!/targets/.+$!/targets!;
	my $local_sys_incl=$Ini{LOCAL_SYS_INCL}; $local_sys_incl=~s/\\/\//g;
	my $shared_host_dir=$Ini{SHARED_HOST_DIR}; $shared_host_dir=~s/\\/\//g;
	my $exptrtver=$VERSION;$trtver=~s/\./\\./g; 
	my $expRemote_TDP_dir='.*/\.?TestRealTime/'.$exptrtver.'/targets';

	open(IN,"<$in");
	open(OUT,">$out");
	while(<IN>)
	{
		chomp;
		if (/^#(line| \d)/) {
			s/"$expRemote_TDP_dir/"$targetsdir/;		# map TDP target dir if needed
			s/"$Ini{SHARED_REMOTE_DIR}/"$shared_host_dir/;	# map remote 'user' dir to local share
#			s/"$Ini{REMOTE_SYS_INCL}/"$local_sys_incl/;	# map remote 'sys' dir to local share
		}
		print OUT "$_\n";
	}
	close(IN);close(OUT);
}

#used for remote compilation
sub Remote_fixana #($$)	# used by postGen: rewrites given input analyser definition file
			# with local include references added to existing remote include references
{
	my ($pattern,$in)=@_;
	my $out = "$in.$$";
	my $local_sys_incl=$Ini{LOCAL_SYS_INCL};
	my $remote_sys_incl=$Ini{REMOTE_SYS_INCL};$remote_sys_incl=~s![\\\/]+!\[\\\\\\\/\]!g;
	open(IN,"<$in");
	open(OUT,">$out");
	while(<IN>)
	{
		chomp;
		print OUT "$_\n";		# rewrite existing line
		if (/^$pattern/
		    && s/$remote_sys_incl/$local_sys_incl/){		# map remote 'sys' dir to local share
			if ( $^O=~/Win32/i ) {
				s/\//\\/g;				# map remaining '/'s to '\'s
			}
			print OUT "$_\n";	# rewrite added modified line
		}
	}
	close(IN);close(OUT);

	rename ("$out","$in");
}

sub WaitFromUser
{
print "WaitFromUser\n a windows is openned\ntype exit into this window when you have finished to execute or debug\n";
my $cmd="";
	if ($^O =~/Win32/i){
		$cmd = "$ENV{ComSpec} /c start /wait \"WaitFromUser\" " ;
	}else{
		$cmd = "sh ";
	}
	$cmd .= "echo type exit when your have finished";
	system($cmd);

}

#REMOTE_CONNECTION="telnet.pl -l _user_ -pw _PassWd_ [-cp /bash\$ $/] "
#The default prompt is '/[\$%#>] $/'
sub telnet #(@)
{
#to use the telnet protocol the following line must be uncommented and Telnet.pm must be installed into .../TestRealTime/lib/perl/Net
# download from http://search.cpan.org/~jrogers/Net-Telnet-3.04  load file and copy /lib/Net/Telnet.pm
#UNCOMMENT
#use Net::Telnet;
   my @argv=@_;
   my $remote_host=shift(@argv);
   my $remote_user="";
   my $remote_pass="";
   my $remote_port=0;
   my $commandPrompt;
   my @commands=();
   my $ret;

   while(@argv){
   	$_=shift(@argv);
   	   if(/^-l$/) {$remote_user=shift(@argv);}
   	elsif(/^-pw$/){$remote_pass=shift(@argv);}
   	elsif(/^-cp$/){$commandPrompt=shift(@argv);}
   	elsif(/^-port$/){$remote_port=shift(@argv);}
   	else{push(@commands,$_);}
   }
   print "##telnet $remote_host\n";
   my $s = Net::Telnet->new() || die "Socket die $@\n";
   $s->prompt($commandPrompt) if($commandPrompt);
   $s->port($remote_port) if($remote_port);
   $ret=$s->open($remote_host);
   print "##connected ($ret)\n";
   print "##login($remote_user, $remote_pass)\n";
   $ret=$s->login($remote_user, $remote_pass);
   print "##logged in ($ret)\n";
   foreach ( @commands) {
	print ">$_\n";
	my @out = $s->cmd($_) ;
	print @out;
   }
   $s->close;
   print "\n##telnet closed\n";
   return 0;
}

#sub tdpRemote_Shell # given as example to be part of the TDP < build settings > additional perl functions when REMOTE_CONNECTION = tdpRemote_Shell
#{
#   use Net::Telnet;
#   my $cmdList=shift;# this is a pointer to the array of cmd to execute.
#   my $remote_host=Remote_Host();
#   my $remote_pass=Remote_PassWd();
#   my $port=1800;
#   my $regularPrompt='/# ?$/';
#
#   my $ret=0;
#   print "##telnet $remote_host $port\n";
#   my $s = Net::Telnet->new(port => $port) || die "Socket die $@\n";
#   $s->errmode("return");
#   $s->prompt($regularPrompt);
#   $ret=$s->open($remote_host);
#   if($ret){
#	print "##connected ($ret)\n";
#	$s->waitfor('/password: ?$/i');
#	$s->print($remote_pass);
#	print "##logged in\n";
#	foreach ( @$cmdList) {
#	print "$_\n";
#	my @out = $s->cmd($_) ;
#	print @out;
#	}
#	$s->close;
#	print "##telnet closed\n";
#   } else { print "##!!!open failed\n";}
#   return 0;
#}

1;

#Questions/Mail to customer, to validate the environment and make sure these atl_execX apply to described environment.
#for execution on the remote QNX or Linux or Unix machine we need to know How the customer send the application on the remote machine.
#actually we prepared 3 possible ways

#atl_exec1 )
#host and remote share the project directory as disk space. So the remote machine can access to the same project directory and execute the binary.
#then a remote connection can order to execute the binary (after few path translations)

#atl_exec2 )
#host and remote share a fixed work directory as disk space. So the host machine copy the binary to the shared directory
#then a remote connection can order to execute the binary from the fixed shared directory.

#atl_exec3 )
#host and remote does not share any disk space. So FTP must be used to send binary on the remote machine into the fixed directory.
#then a remote connection can order to execute the binary from this directory.

#If none of the above execution mode is used we need to know: step by step (like a batch/shell file) what the customer does, to execute on the remote machine,
#to be able to include each step into the TDP execution function in order to automate.
