#! perl -w

# Licensed Materials - Property of HCL and/or IBM
# Copyright HCL Technologies Ltd. 2016, 2020. All Rights Reserved.
# Copyright IBM Corporation 1999, 2016. All Rights Reserved.
#
# U.S. Government Users Restricted Rights -  Use, duplication or 
# disclosure restricted by GSA ADP Schedule.

# Win9x command.com does not interact with perl according to win32 standard.
# We therefore need to call some functions in a non-standard way.
#
# This detects if the system is Win95 for that purpose.

# Miscellaneous functions
use File::Copy;

my $VERBOSE_MODE=0; #$ENV{ATTOLSTUDIO_VERBOSE};
my $targetfilename;
$TARGETDIR='';

sub mySystem {
    my( $OS, $ROSERT_HOST );

    $OS          = '?' unless $OS          = $ENV{'OS'};
    $ROSERT_HOST = '?' unless $ROSERT_HOST = $ENV{'ROSERT_HOST'};
    if ($VERBOSE_MODE) {
        print STDERR "====================================\n";
        print STDERR "system call @_\n";
        print STDERR "location : \n";
        system("pwd");
        print STDERR "====================================\n";
    }
    if(($ROSERT_HOST eq 'win32') && ($OS !~ m{^Windows_NT$}i )) {
        return system( join( ' ', @_ ) );
    } else {
        return system( @_ );
    }
}

my $status = 0;
my @command;
my $localTestrtEnvFile = 'envTestRTRtistcc.pl';


if (-e $localTestrtEnvFile) {
    require $localTestrtEnvFile;
} else {
    print $localTestrtEnvFile." not found: create a OneTestEmbedded project and save settings before compiling...\n";
    die;
}

push @command, "trtld";
push @command, "-config";
push @command, $localTestrtEnvFile;

if ($VERBOSE_MODE) {
    print STDERR "=============== LD raw =================\n";
    foreach $arg ( @ARGV ) {
        print STDERR "\t".$arg."\n";
    }
    print STDERR "====================================\n";
}
my @postcommand;
my @library;
my @libpath;
my @objects;
my @targetexe;

if (($^O=~/win32/i) && ($TARGETDIR=~/visual/i)) {
    if ($VERBOSE_MODE) {
        print STDERR "visual mode\n";
    }
    while(@ARGV) {
        $_=shift(@ARGV);
        if (/^\/Fo(.*)$/) {
            # convert /Fo into -o    
            push @targetexe, "-o";
            push @targetexe, $1;
        } elsif (/^\/OUT:(.*)/) {
            # convert /out into -o    
            push @targetexe, "-o";
            push @targetexe, $1;
        } elsif(/^-l(.*)$/){
            if ($1 =~ m/(.*)\.lib/i) {
               push @library, $1;
            } else {
                push @library, $1.".lib";
            }
        } elsif(/^-L\"?(.*)/){
            my $p=$1;$p=~s/\"$//;   
            push @libpath, "-L".$p;
        } elsif (/(.*)\.LIB\"?$/i) {
            push @library, $_;
        } elsif (/(.*)\.OBJ\"?$/i) {
            push @objects, $_;
        } else {
            push @postcommand, $_;
        }
    }
    push @command, @objects;
    push @command, @library;
    push @command, @targetexe;
    push @command, @libpath;
    if (@postcommand != 0) {
        push @command, "--";
        push @command, @postcommand;
    }
    if ($VERBOSE_MODE) {
        print STDERR "=============== LD adapted =================\n";
        foreach $arg ( @command ) {
            print STDERR "\t".$arg."\n";
        }
        print STDERR "====================================\n";
    }
} else {
    foreach $arg ( @ARGV ) {    
        push @command, $arg;
    }
}



if( $status == 0 ) {
    $status = &mySystem( @command );
}

exit 0;


